@@ -1,4 +1,4 @@
-package EAI::Common 0.9;
+package EAI::Common 1.0;
 
 use strict; use feature 'unicode_strings'; use warnings; no warnings 'uninitialized';
 use Exporter qw(import); use EAI::DateUtil qw(get_curdate is_holiday); use Data::Dumper qw(Dumper); use Getopt::Long qw(:config no_ignore_case); use Log::Log4perl qw(get_logger); use MIME::Lite (); use Scalar::Util qw(looks_like_number); use Module::Refresh ();
@@ -18,19 +18,19 @@ my %hashCheck = (
 		task => {},
 	},
 	config => { # parameter category for site global settings, defined in site.config and other associated configs loaded at INIT
-		checkLookup => {"test.pl" => {errmailaddress => "",errmailsubject => "",timeToCheck =>"", freqToCheck => "", logFileToCheck => "", logcheck => "",logRootPath =>""}}, # used for logchecker, each entry of the hash defines a log to be checked, defining errmailaddress to receive error mails, errmailsubject, timeToCheck as earliest time to check for existence in log, freqToCheck as frequency of checks (daily/monthly/etc), logFileToCheck as the name of the logfile to check, logcheck as the regex to check in the logfile and logRootPath as the folder where the logfile is found. lookup key: $execute{scriptname} + $execute{addToScriptName}
+		checkLookup => {}, # ref to datastructure {"scriptname.pl" => {errmailaddress => "",errmailsubject => "",timeToCheck =>"", freqToCheck => "", logFileToCheck => "", logcheck => "",logRootPath =>""},...} used for logchecker, each entry of the hash lookup table defines a log to be checked, defining errmailaddress to receive error mails, errmailsubject, timeToCheck as earliest time to check for existence in log, freqToCheck as frequency of checks (daily/monthly/etc), logFileToCheck as the name of the logfile to check, logcheck as the regex to check in the logfile and logRootPath as the folder where the logfile is found. lookup key: $execute{scriptname} + $execute{addToScriptName}
 		errmailaddress => "", # default mail address for central logcheck/errmail sending 
 		errmailsubject => "", # default mail subject for central logcheck/errmail sending 
 		executeOnInit => "", # code to be executed during INIT of EAI::Wrap to allow for assignment of config/execute parameters from commandline params BEFORE Logging!
-		folderEnvironmentMapping => {Test => "Test", Dev => "Dev", "" => "Prod"}, # Mapping for $execute{envraw} to $execute{env}:
+		folderEnvironmentMapping => {}, # ref to hash {Test => "Test", Dev => "Dev", "" => "Prod"}, mapping for $execute{envraw} to $execute{env}
 		fromaddress => "", # from address for central logcheck/errmail sending, also used as default sender address for sendGeneralMail
-		historyFolder => {"" => "default"}, # folders where downloaded files are historized, lookup key as checkLookup, default in "" =>
-		historyFolderUpload => {"" => "default"}, # folders where uploaded files are historized, lookup key as checkLookup, default in "" =>
+		historyFolder => {}, # ref to hash {"scriptname.pl" => "folder"}, folders where downloaded files are historized, lookup key as in checkLookup, default in "" => "defaultfolder"
+		historyFolderUpload => {}, # ref to hash {"scriptname.pl" => "folder"}, folders where uploaded files are historized, lookup key as in checkLookup, default in "" => "defaultfolder"
 		logCheckHoliday => "", # calendar for business days in central logcheck/errmail sending. builtin calendars are AT (Austria), TG (Target), UK (United Kingdom) and WE (for only weekends). Calendars can be added with EAI::DateUtil::addCalendar
 		logs_to_be_ignored_in_nonprod => '', # logs to be ignored in central logcheck/errmail sending
-		logRootPath => {"" => "default"}, # paths to log file root folders (environment is added to that if non production), lookup key as checkLookup, default in "" =>
-		redoDir => {"" => "default"}, # folders where files for redo are contained, lookup key as checkLookup, default in "" =>
-		sensitive => {"prefix" => {user=>"",pwd =>"",hostkey=>"",privkey =>""}}, # hash lookup for sensitive access information in DB and FTP (lookup keys are set with DB{prefix} or FTP{prefix}), may also be placed outside of site.config; all sensitive keys can also be environment lookups, e.g. hostkey=>{Test => "", Prod => ""} to allow for environment specific setting
+		logRootPath => {}, # ref to hash {"scriptname.pl" => "folder"}, paths to log file root folders (environment is added to that if non production), lookup key as checkLookup, default in "" => "defaultfolder"
+		redoDir => {}, # ref to hash {"scriptname.pl" => "folder"}, folders where files for redo are contained, lookup key as checkLookup, default in "" => "defaultfolder"
+		sensitive => {}, # hash lookup table ({"prefix" => {user=>"",pwd =>"",hostkey=>"",privkey =>""},...}) for sensitive access information in DB and FTP (lookup keys are set with DB{prefix} or FTP{prefix}), may also be placed outside of site.config; all sensitive keys can also be environment lookups, e.g. hostkey=>{Test => "", Prod => ""} to allow for environment specific setting
 		smtpServer => "", # smtp server for den (error) mail sending
 		smtpTimeout => 60, # timeout for smtp response
 		testerrmailaddress => '', # error mail address in non prod environment
@@ -94,15 +94,15 @@ my %hashCheck = (
 		longreadlen => 1024, # used for setting database handles LongReadLen parameter for DB connection, if not set defaults to 1024
 		noDBTransaction => 1, # don't use a DB transaction for dumpDataIntoDB
 		noDumpIntoDB => 1, # if files from this load should not be dumped to the database
-		postDumpExecs => [{execs => ['',''], condition => ''}], # done in dumpDataIntoDB after postDumpProcessing and before commit/rollback. doInDB everything in execs if condition is fulfilled 
-		postDumpProcessing => "", # done in dumpDataIntoDB after storeInDB, execute perl code in postDumpProcessing
-		postReadProcessing => "", # done in writeFileFromDB after readFromDB, execute perl code in postReadProcessing
-		prefix => "db", # key for sensitive information (e.g. pwd and user) in config{sensitive}
+		postDumpExecs => [], # array for execs done in dumpDataIntoDB after postDumpProcessing and before commit/rollback: [{execs => ['',''], condition => ''}]. doInDB all execs if condition (evaluated string or anonymous sub: condition => sub {...}) is fulfilled
+		postDumpProcessing => "", # done in dumpDataIntoDB after storeInDB, execute perl code in postDumpProcessing (evaluated string or anonymous sub: postDumpProcessing => sub {...})
+		postReadProcessing => "", # done in writeFileFromDB after readFromDB, execute perl code in postReadProcessing (evaluated string or anonymous sub: postReadProcessing => sub {...})
+		prefix => "", # key for sensitive information (e.g. pwd and user) in config{sensitive}
 		primkey => "", # primary key indicator to be used for update statements, format: "key1 = ? AND key2 = ? ..."
 		pwd => "", # for password setting, either directly (insecure -> visible) or via sensitive lookup
 		query => "", # query statement used for readFromDB and readFromDBHash
 		schemaName => "", # schemaName used in dumpDataIntoDB/storeInDB, if tableName contains dot the extracted schema from tableName overrides this. Needed for datatype information!
-		server => {Prod => "", Test => ""}, # DB Server in environment hash
+		server => {}, # DB Server in environment hash lookup: {Prod => "", Test => ""}
 		tablename => "", # the table where data is stored in dumpDataIntoDB/storeInDB
 		upsert => 1, # in dumpDataIntoDB/storeInDB, should an update be done after the insert failed (because of duplicate keys) or insert after the update failed (because of key not exists)?
 		user => "", # for user setting, either directly (insecure -> visible) or via sensitive lookup
@@ -116,9 +116,9 @@ my %hashCheck = (
 		emptyOK => 0, # flag to specify whether empty files should not invoke an error message. Also needed to mark an empty file as processed in EAI::Wrap::markProcessed
 		extract => 1, # flag to specify whether to extract files from archive package (zip)
 		extension => "", # the extension of the file to be read (optional, used for redoFile)
-		fieldCode => {}, # additional field based processing code: fieldCode => {field1 => 'perl code', ..}, invoked if key equals either header (as in format_header) or targetheader (as in format_targetheader) or invoked for all fields if key is empty {"" => 'perl code'}. set $skipLineAssignment to true (1) if current line should be skipped from data.
+		fieldCode => {}, # additional field based processing code: fieldCode => {field1 => 'perl code', ..}, invoked if key equals either header (as in format_header) or targetheader (as in format_targetheader) or invoked for all fields if key is empty {"" => 'perl code'}. set $EAI::File::skipLineAssignment to true (1) if current line should be skipped from data. perl code can be an evaluated string or an anonymous sub: field1 => sub {...}
 		filename => "", # the name of the file to be read
-		firstLineProc => '', # processing done in reading the first line of text files
+		firstLineProc => "", # processing done in reading the first line of text files
 		format_allowLinefeedInData => 1, # line feeds in values don't create artificial new lines/records, only works for csv quoted data
 		format_beforeHeader => "", # additional String to be written before the header in write text
 		format_dateColumns => [], # numeric array of columns that contain date values (special parsing) in excel files
@@ -147,7 +147,7 @@ my %hashCheck = (
 		format_xlformat => "xlsx|xls", # excel format for parsing, also specifies excel parsing
 		format_xpathRecordLevel => "", # xpath for level where data nodes are located in xml
 		format_XML => 1, # specify xml parsing
-		lineCode => "", # additional line based processing code, invoked after whole line has been read
+		lineCode => "", # additional line based processing code, invoked after whole line has been read (evaluated string or anonymous sub: lineCode => sub {...})
 		localFilesystemPath => "", # if files are taken from or put to the local file system with getLocalFiles/putFileInLocalDir then the path is given here. Setting this to "." avoids copying files.
 		optional => 1, # to avoid error message for missing optional files, set this to 1
 	},
@@ -171,9 +171,9 @@ my %hashCheck = (
 		privKey => "", # sftp key file location for Net::SFTP::Foreign, either directly (insecure -> visible) or via sensitive lookup
 		pwd => "", # for password setting, either directly (insecure -> visible) or via sensitive lookup
 		queue_size => 1, # queue_size for Net::SFTP::Foreign, if > 1 this causes often connection issues
-		remove => {removeFolders=>[], day=>, mon=>, year=>1}, # for for removing (archived) files with removeFilesOlderX, all files in removeFolders are deleted being older than day=> days, mon=> months and year=> years
+		remove => {}, # ref to hash {removeFolders=>[], day=>, mon=>, year=>1} for for removing (archived) files with removeFilesOlderX, all files in removeFolders are deleted being older than day days, mon months and year years
 		remoteDir => "", # remote root folder for up-/download, archive and remove: "out/Marktdaten/", path is added then for each filename (load)
-		remoteHost => {Prod => "", Test => ""}, # ref to hash of IP-addresses/DNS of host(s).
+		remoteHost => {}, # ref to hash of IP-addresses/DNS of host(s).
 		SFTP => 0, # to explicitly use SFTP, if not given SFTP will be derived from existence of privKey or hostkey
 		simulate => 0, # for removal of files using removeFilesinFolderOlderX/removeFilesOlderX only simulate (1) or do actually (0)?
 		sshInstallationPath => "", # path were ssh/plink exe to be used by Net::SFTP::Foreign is located
@@ -197,26 +197,30 @@ my %hashCheck = (
 		customHistoryTimestamp => "", # optional custom timestamp to be added to filenames moved to History/HistoryUpload/FTP archive, if not given, get_curdatetime is used (YYYYMMDD_hhmmss)
 		execOnly => "", # used to remove loads where $common{task}{execOnly} !~ $load->{process}{onlyExecFor}
 		ignoreNoTest => 0, # ignore the notest file in the process-script folder, usually preventing all runs that are not in production
-		plannedUntil => "2359", # latest time that planned repitition should last
+		plannedUntil => "", # latest time that planned repitition should last
 		redoFile => 1, # flag for specifying a redo
 		redoTimestampPatternPart => "", # part of the regex for checking against filename in redo with additional timestamp/redoDir pattern (e.g. "redo", numbers and _), anything after files barename (and before ".$ext" if extension is defined) is regarded as a timestamp. Example: '[\d_]', the regex is built like ($ext ? qr/$barename($redoTimestampPatternPart|$redoDir)*\.$ext/ : qr/$barename($redoTimestampPatternPart|$redoDir)*.*/)
 		retrySecondsErr => 60, # retry period in case of error
 		retrySecondsErrAfterXfails => 600, # after fail count is reached this alternate retry period in case of error is applied. If 0/undefined then job finishes after fail count
 		retrySecondsXfails => 3, # fail count after which the retrySecondsErr are changed to retrySecondsErrAfterXfails
 		retrySecondsPlanned => 300, # retry period in case of planned retry
-		skipHolidays => 0, # skip script execution on holidays
-		skipHolidaysDefault => "AT", # holiday calendar to take into account for skipHolidays
+		skipHolidays => "", # skip script execution on holidays
+		skipHolidaysDefault => "", # holiday calendar to take into account for skipHolidays
 		skipWeekends => 0, # skip script execution on weekends
-		skipForFirstBusinessDate => 0, # used for "wait with execution for first business date", either this is a calendar or 1 (then calendar is skipHolidaysDefault), this cannot be used together with skipHolidays
+		skipForFirstBusinessDate => "", # used for "wait with execution for first business date", either this is a calendar or 1 (then calendar is skipHolidaysDefault), this cannot be used together with skipHolidays
 	},
 );
-# ignore type checking for these as they might have different types
-my %ignoreType = (
-	Filecolumns => 1, # can be ref to hash or ref to array
-	Fileformat_skip => 1, # can be line number (int) or "skip until pattern" string
-	Fileformat_sep => 1, # can be regex split or separator string
-	taskskipHolidays => 1, # can be true (1) or calendar string
-	taskskipForFirstBusinessDate => 1, # can be true (1) or calendar string
+# alternate type checking for these as they might have different types
+my %alternateType = (
+	configexecuteOnInit => sub {}, # can be eval string or anonymous sub
+	DBpostDumpProcessing => sub {}, # can be eval string or anonymous sub
+	DBpostReadProcessing => sub {}, # can be eval string or anonymous sub
+	Filecolumns => [], # can be ref to hash or ref to array
+	Fileformat_skip => 1, # can be "skip until pattern" string or line number (int)
+	Fileformat_sep => qr//, # can be separator string or regex split
+	FilelineCode => sub {}, # can be eval string or anonymous sub
+	taskskipHolidays => 1, # can be calendar string or true (1)
+	taskskipForFirstBusinessDate => 1, # can be calendar string or true (1)
 );
 
 our %common;our %config;our @loads;our %execute;our @optload;our %opt;
@@ -389,8 +393,23 @@ sub checkHash ($$) {
 				} else {
 					# check type for existing keys, if explicitly not defined then ignore...
 					if (defined($hash->{$defkey})) {
-						die "wrong reference type for value: \$".$hashName."{".$defkey."}: ".ref($hashCheck{$hashName}{$defkey})." not like passed:".ref($hash->{$defkey}).",".$locStr if (ref($hashCheck{$hashName}{$defkey}) ne ref($hash->{$defkey}) && !$ignoreType{$hashName.$defkey});
-						die "wrong type for value: \$".$hashName."{".$defkey."},".$locStr if (looks_like_number($hashCheck{$hashName}{$defkey}) ne looks_like_number($hash->{$defkey}) && !$ignoreType{$hashName.$defkey});
+						if (ref($hashCheck{$hashName}{$defkey}) ne ref($hash->{$defkey})) {
+							if (!defined($alternateType{$hashName.$defkey}) or (ref($alternateType{$hashName.$defkey}) ne ref($hash->{$defkey}))) {
+								die "wrong reference type for value: \$".$hashName."{".$defkey."}:".ref($hash->{$defkey}).", it should be either:".ref($hashCheck{$hashName}{$defkey})." or:".ref($alternateType{$hashName.$defkey}).",".$locStr;
+							}
+						}
+						# numeric check: either hashcheck is same looks_like_number as given key or alternateType is same looks_like_number
+						if (looks_like_number($hash->{$defkey}) and !looks_like_number($hashCheck{$hashName}{$defkey})) {
+							if (!defined($alternateType{$hashName.$defkey}) or (looks_like_number($hash->{$defkey}) and !looks_like_number($alternateType{$hashName.$defkey}))) {
+								die "wrong numeric type for value: \$".$hashName."{".$defkey."},".$locStr;
+							}
+						}
+						# non-numeric check: if non-numeric type given, then check if either hashcheck looks like a number or hashcheck was a ref type and alternateType looks like a number
+						if (!looks_like_number($hash->{$defkey}) and !ref($hash->{$defkey}) and looks_like_number($hashCheck{$hashName}{$defkey})) {
+							if (!defined($alternateType{$hashName.$defkey}) or (!looks_like_number($hash->{$defkey}) and looks_like_number($alternateType{$hashName.$defkey}))) {
+								die "wrong non-numeric type for value: \$".$hashName."{".$defkey."},".$locStr;
+							}
+						}
 					}
 				}
 			}
@@ -447,14 +466,18 @@ sub setupLogging {
 	# get logRootPath, historyFolder, historyFolderUpload and redoDir from lookups in config.
 	# if they are not in the script home directory (having an absolute path) then build environment-path separately for end folder (script home directory is already in its environment)
 	for my $foldKey ("redoDir","logRootPath","historyFolder","historyFolderUpload") {
-		my $folder = $config{$foldKey}{$execute{scriptname}.$execute{addToScriptName}};
+		warn ("\$config{$foldKey} is not a hash, cannot get $foldKey from it !") if (ref($config{$foldKey}) ne "HASH");
+		my $folder = $config{$foldKey}{$execute{scriptname}.$execute{addToScriptName}} if (ref($config{$foldKey}) eq "HASH");
 		my $defaultFolder;
 		if (!$folder) {
 			$folder = $config{$foldKey}{""}; # take default, if no lookup defined for script
 			$defaultFolder = 1;
 		}
-		if ($folder =~ /^(\S:)*[\\|\/](.*?)$/) {
-			my ($folderPath,$endFolder) = ($folder =~ /(.*)[\\|\/](.*?)$/); # both slash as well as backslash act as path separator for last part
+		$folder = $opt{config}{$foldKey} if $opt{config}{$foldKey}; # also possible to override these settings via options
+		# convert backslash to slash to avoid problems in string handling (unicode ...)
+		$folder =~ s/\\/\//g;
+		if ($folder =~ /^(\S:)*\/(.*?)$/ and !$opt{config}{$foldKey}) {
+			my ($folderPath,$endFolder) = ($folder =~ /(.*)\/(.*?)$/); # slash acts as path separator for last part
 			# default Folder is built differently: folderPath/endFolder/environ instead of folderPath/environ/endFolder
 			$execute{$foldKey} = ($defaultFolder ? $folderPath."/".$endFolder.($execute{envraw} ? "/".$execute{envraw} : "") : $folderPath.($execute{envraw} ? "/".$execute{envraw} : "")."/".$endFolder);
 		} else {
@@ -492,7 +515,10 @@ sub setupLogging {
 			Log::Log4perl->appenders()->{"MAIL"}->{"appender"}->{"to"} = [$config{testerrmailaddress}] if $config{testerrmailaddress};
 			$logger->error("no errmailaddress found for ".$execute{scriptname}.$execute{addToScriptName}.", no entry found in \$config{checkLookup}{$execute{scriptname}.$execute{addToScriptName}}");
 		}
-		setErrSubject("Setting up EAI.Wrap"); # general context after logging initialization: setup of EAI.Wrap by script
+		setErrSubject("Setting up EAI::Wrap"); # general context after logging initialization: setup of EAI::Wrap by script
+	} else {
+		# remove any defined mail appenders
+		undef(Log::Log4perl->appenders()->{"MAIL"}) if Log::Log4perl->appenders()->{"MAIL"};
 	}
 }
 
@@ -505,30 +531,31 @@ sub setupEAIWrap {
 	$logger->info("started $execute{scriptname} in $execute{homedir} (environment $execute{env}) ... execute parameters: ".dumpFlat(\%execute,1,1)." ... common parameters: ".dumpFlat(\%common,1,1));
 	if ($logger->is_debug) {
 		$logger->debug("load $_ parameters: ".dumpFlat($loads[$_],1,1)) for (0..$#loads);
+		$logger->trace("config parameters: ".dumpFlat(\%config,1,1)) if !defined($config{sensitive}) and $logger->is_trace();
 	}
 	# check starting conditions and exit if met (returned true)
 	checkStartingCond(\%common) and exit 0;
-	setErrSubject("General EAI.Wrap script execution"); # general context after setup of EAI.Wrap
+	setErrSubject("General EAI::Wrap script execution"); # general context after setup of EAI::Wrap
 }
 
 # returned Data::Dumpered datastructure given in $arg flattened, sorted (if $sortDump given) and compressed (if $compressDump given)
-sub dumpFlat {
+sub dumpFlat ($;$$) {
 	my $arg = shift;
 	my $sortDump = shift;
 	my $compressDump = shift;
 	$Data::Dumper::Indent = 0; # temporarily flatten dumper output for single line
 	$Data::Dumper::Sortkeys = 1 if $sortDump; # sort keys to get outputs easier to read
 	my $dump = Dumper($arg);
-	$dump =~ s/\$VAR1=//;
 	if ($compressDump) {
 		$dump =~ s/\s+//g;$dump =~ s/,'/,/g;$dump =~ s/{'/{/g;$dump =~ s/'=>/=>/g; # compress information
 	}
+	$dump =~ s/\$VAR1=//;
 	$Data::Dumper::Indent = 2;
 	return $dump;
 }
 
 # refresh modules and logging config for changes
-sub refresh() {
+sub refresh {
 	# refresh modules to enable correction of processing without restart
 	Module::Refresh->refresh;
 	# also check for changes in logging configuration
@@ -584,6 +611,7 @@ sub checkStartingCond ($) {
 sub sendGeneralMail ($$$$$$;$$$$) {
 	my ($From, $To, $Cc, $Bcc, $Subject, $Data, $Type, $Encoding, $AttachType, $AttachFile) = @_;
 	my $logger = get_logger();
+	$logger->error("cannot send mail as \$config{smtpServer} not set") if !$config{smtpServer};
 	$logger->info("sending general mail From:".($From ? $From : $config{fromaddress}).", To:".($execute{envraw} ? $config{testerrmailaddress} : $To).", CC:".($execute{envraw} ? "" : $Cc).", Bcc:".($execute{envraw} ? "" : $Bcc).", Subject:".($execute{envraw} ? $execute{envraw}.": " : "").$Subject.", Type:".($Type ? $Type : "TEXT").", Encoding:".($Type eq 'multipart/related' ? undef : $Encoding).", AttachType:$AttachType, AttachFile:$AttachFile ...");
 	$logger->debug("Mailbody: $Data");
 	my $msg = MIME::Lite->new(
@@ -618,7 +646,7 @@ sub sendGeneralMail ($$$$$$;$$$$) {
 			Path => $AttachFile
 		);
 	}
-	$msg->send('smtp', $config{smtpServer}, AuthUser=>$config{smtpAuth}{user}, AuthPass=>$config{smtpAuth}{pwd});
+	$msg->send();
 	if ($msg->last_send_successful()) {
 		$logger->info("Mail sent");
 		$logger->trace("sent message: ".$msg->as_string) if $logger->is_trace();
@@ -642,7 +670,7 @@ sub send_email {
 			Type    => "TEXT",
 			Data    => $p{message},
 		);
-	$msg->send('smtp', $config{smtpServer}, AuthUser=>$config{smtpAuth}{user}, AuthPass=>$config{smtpAuth}{pwd});
+	eval {$msg->send();} or warn("couldn't send error mail: $@");
 }
 
 1;
@@ -659,20 +687,22 @@ EAI::Common - Common parts for the EAI::Wrap package
  @loads .. list of hashes defining specific load processes
  %execute .. hash of parameters for current running task script
 
- readConfigFile
- getSensInfo
- setupConfigMerge
- getOptions
- extractConfigs
- checkHash
- setupEAIWrap
- getLogFPathForMail
- getLogFPath
- MailFilter
- setupLogging
- setErrSubject $context
- checkStartingCond $process
- sendGeneralMail $From, $To, $Cc, $Bcc, $Subject, $Type, $Data, $Encoding, $AttachType, $AttachFile
+ readConfigFile ($configfilename)
+ getSensInfo ($prefix, $key)
+ setupConfigMerge ()
+ getOptions ()
+ extractConfigs ($contextSub, $arg, @required)
+ checkHash ($hash, $hashName)
+ checkParam ($subhash, $hashName)
+ setupEAIWrap ()
+ dumpFlat ($arg, $sortDump, $compressDump)
+ getLogFPathForMail ()
+ getLogFPath ()
+ MailFilter ()
+ setErrSubject ($context)
+ setupLogging ()
+ checkStartingCond ($task)
+ sendGeneralMail ($From, $To, $Cc, $Bcc, $Subject, $Type, $Data, $Encoding, $AttachType, $AttachFile)
 
 =head1 DESCRIPTION
 
@@ -688,7 +718,9 @@ read given config file (eval perl code in site.config and related files)
 
 =item getSensInfo ($$)
 
-get sensitive info from $config{sensitive}{$prefix}{$key}, arguments are $prefix and $key, depending on queried key being a ref to hash, get the environment lookup hash key value or the value directly
+arguments are $prefix and $key
+
+get sensitive info from $config{sensitive}{$prefix}{$key}, depending on queried key being a ref to hash, get the environment lookup hash key value or the value directly
 
 =item setupConfigMerge
 
@@ -706,15 +738,21 @@ get options for overriding configured settings, results are stored in globally a
 
 =item extractConfigs ($$$;@)
 
-sets error subject to $contextSub (first argument) and extracts config hashes (DB,FTP,File,process,task) from ref to hash $arg (second argument) and return them as a list of hashes. The config hashes to be extracted are given as strings in the following parameter list @required (at least one is required).
+arguments are $contextSub, $arg and @required.
+
+sets error subject to $contextSub (first argument) and extracts config hashes (DB,FTP,File,process,task) from ref to hash $arg (second argument) and return them as a list of hashes. The config hashes to be extracted are given as strings in the parameter list @required (at least one is required).
 
 =item checkHash ($$)
 
-check config hash passed in $hash for validity against hashCheck (valid key entries are there + their valid value types (examples)). returns 0 on error and exception $@ contains details, to allow for checkHash(..) or {handle exception}
+arguments are $hash and $hashName
+
+check keys of config subhash (named $hashName) being passed in $hash for validity against internal hash %hashCheck (valid key entries + their valid value types (examples) are defined there). returns 0 on error and exception $@ contains details, to allow for checkHash(..) or {handle exception}.
 
 =item checkParam ($$)
 
-check parameter key from second argument within first argument $subhash, returns 0 if not defined or not existing (only non-numerics)
+arguments are $subhash and $hashName
+
+check existence of parameter $hashName within first argument $subhash, returns 0 if not defined or empty (only non-numerics)
 
 =item setupEAIWrap
 
@@ -724,6 +762,12 @@ Correctness of the configuration and starting conditions are also checked, preve
 
 following three functions can be used in the central log.config as coderefs for callback.
 
+=item dumpFlat ($;$$)
+
+arguments are $arg: datastructure to be flat dumped, $sortDump: sort all keys in $arg and $compressDump: compress information in $arg
+
+returns Data::Dumper dumped datastructure given in $arg flattened, sorted (if $sortDump given) and compressed (if $compressDump given)
+
 =item getLogFPathForMail
 
 for custom conversion specifiers: returns path of configured logfile resp logfile of previous day (as a file:// hyperlink)
@@ -738,9 +782,9 @@ for Mail appender config: used for filtering if further mails should be sent, co
 
 =item setErrSubject ($)
 
-set context specific subject for ErrorMail
+argument $context .. text for context of subject
 
- $context .. text for context of subject
+set context specific subject for ErrorMail
 
 =item setupLogging
 
@@ -752,12 +796,26 @@ Each entry of the sub-hash defines defines the errmailaddress to receive error m
 
 =item checkStartingCond ($)
 
+argument $task .. config information
+
 check starting conditions from process config information and return 1 if met
 
- $task .. config information
-
 =item sendGeneralMail ($$$$$$;$$$$)
 
+arguments are
+
+ $From .. sender
+ $To .. recipient
+ $Cc .. cc recipient (optional, but need arg)
+ $Bcc .. mcc recipient  (optional, but need arg)
+ $Subject .. mail subject
+ $Data .. the mail body, either plain text or html.
+ $Type .. mail mime type (eg text/plain, text/html oder 'multipart/related'), if 'multipart/related', then a ref to array to the filenames (path), that should be attached is expected to be set in $AttachFile. 
+ In the above example a mail body ($Data) is being set as the first attachment and its type is text/html. The rest of the attachments from $AttachFile are encoded using base64 and all have mime type AttachType (see below).
+ $Encoding .. encoding for mail body, optional (eg quoted-printable)
+ $AttachType .. mime type for attachment(s) (eg text/csv or image/png), optional
+ $AttachFile .. file name/path(s) for attachment(s), optional (hat to be ref to array, if $Type = 'multipart/related')
+
 send general mail, either simple text or html mails, mails with an attachment or multipart mails for "in-body" attachments (eg pictures). 
 In this case the mail body needs to be HTML, attachments are referred to inside the HTML code and are passed as a ref to array of paths in $AttachFile.
  
@@ -771,20 +829,6 @@ Example:
  my @filelist = glob("*.png");
  sendGeneralMail(undef,'address@somewhere.com',undef,undef,"subject for mail",$body,'multipart/related','quoted-printable','image/png',\@filelist);
 
-parameters for sendGeneralMail
-
- $From .. sender
- $To .. recipient
- $Cc .. cc recipient (optional, but need arg)
- $Bcc .. mcc recipient  (optional, but need arg)
- $Subject .. mail subject
- $Data .. the mail body, either plain text or html.
- $Type .. mail mime type (eg text/plain, text/html oder 'multipart/related'), if 'multipart/related', then a ref to array to the filenames (path), that should be attached is expected to be set in $AttachFile. 
- In the above example a mail body ($Data) is being set as the first attachment and its type is text/html. The rest of the attachments from $AttachFile are encoded using base64 and all have mime type AttachType (see below).
- $Encoding .. encoding for mail body, optional (eg quoted-printable)
- $AttachType .. mime type for attachment(s) (eg text/csv or image/png), optional
- $AttachFile .. file name/path(s) for attachment(s), optional (hat to be ref to array, if $Type = 'multipart/related')
-
 =back
 
 =head1 COPYRIGHT
