@@ -1,4 +1,4 @@
-package EAI::Wrap 0.9;
+package EAI::Wrap 1.0;
 
 use strict; use feature 'unicode_strings'; use warnings;
 use Exporter qw(import); use Data::Dumper qw(Dumper); use File::Copy qw(copy move); use Cwd qw(chdir); use Archive::Extract ();
@@ -53,8 +53,14 @@ sub INIT {
 		EAI::Common::readConfigFile($_) for sort glob($EAI_WRAP_SENS_CONFIG_PATH."/".$execute{envraw}."/*.config");
 	}
 	EAI::Common::getOptions(); # getOptions before logging setup as centralLogHandling depends on interactive options passed
-	eval $config{executeOnInit} if $config{executeOnInit};
-	die("Error parsing config{executeOnInit} ($config{executeOnInit}): $@") if $@;
+	if ($config{executeOnInit}) {
+		if (ref($config{executeOnInit}) eq "CODE") {
+			eval {$config{executeOnInit}->()};
+		} else {
+			eval $config{executeOnInit};
+		}
+		die("Error parsing config{executeOnInit} ".(ref($config{executeOnInit}) eq "CODE" ? "defined sub" : "'".$config{executeOnInit}."'").": $@") if $@;
+	}
 	$execute{failcount}=0;
 	EAI::Common::setupLogging();
 }
@@ -424,22 +430,21 @@ sub dumpDataIntoDB ($) {
 			}
 			# post processing (Perl code) for config, where postDumpProcessing is defined
 			if ($DB->{postDumpProcessing}) {
-				$logger->info("starting postDumpProcessing");
-				$logger->debug($DB->{postDumpProcessing});
-				eval $DB->{postDumpProcessing};
-				if ($@) {
-					$logger->error("error in eval postDumpProcessing: ".$DB->{postDumpProcessing}.": $@");
-					$hadDBErrors = 1;
-				}
+				$hadDBErrors = evalCustomCode($DB->{postDumpProcessing},"postDumpProcessing");
 			}
 			# post processing (execute in DB!) for all configs, where postDumpExecs conditions and referred execs (DB scripts, that should be executed) are defined
 			if (!$hadDBErrors && $DB->{postDumpExecs}) {
 				$logger->info("starting postDumpExecs ... ");
 				for my $postDumpExec (@{$DB->{postDumpExecs}}) {
 					$logger->info("checking postDumpExec condition: ".$postDumpExec->{condition});
-					my $dopostdumpexec = eval $postDumpExec->{condition};
+					my $dopostdumpexec;
+					if (ref($config{executeOnInit}) eq "CODE") {
+						eval {$dopostdumpexec = eval $postDumpExec->{condition}->();};
+					} else {
+						$dopostdumpexec = eval $postDumpExec->{condition};
+					}
 					if ($@) {
-						$logger->error("error parsing postDumpExec condition: ".$postDumpExec->{condition}.": $@");
+						$logger->error("error parsing postDumpExec condition: ".(ref($postDumpExec->{condition}) eq "CODE" ? "defined sub" : "'".$postDumpExec->{condition}."'").": $@");
 						$hadDBErrors = 1;
 						last;
 					}
@@ -491,6 +496,25 @@ sub dumpDataIntoDB ($) {
 	}
 }
 
+# evaluate custom code contained either in string or ref to sub
+sub evalCustomCode ($$) {
+	my $customCode = shift;
+	my $processingName = shift;
+	my $logger = get_logger();
+	$logger->info("starting $processingName");
+	if (ref($customCode) eq "CODE") {
+		eval {$customCode->()};
+	} else {
+		eval $customCode;
+	}
+	if ($@) {
+		$logger->error("eval of $processingName ".(ref($customCode) eq "CODE" ? "defined sub" : "'".$customCode."'")." returned error:$@");
+		return 0;
+	}
+	return 1;
+}
+
+
 # mark files as being processed depending on whether there were errors, also decide on removal/archiving of downloaded files
 sub markProcessed ($) {
 	my $arg = shift;
@@ -534,8 +558,7 @@ sub writeFileFromDB ($) {
 	$logger->warn("no data retrieved from database for file ".$File->{filename}.", query: ".$DB->{query}) if ($process->{data} and @{$process->{data}} == 0);
 	# prepare for all configs, where postReadProcessing is defined
 	if ($DB->{postReadProcessing}) {
-		eval $DB->{postReadProcessing};
-		$logger->error("error doing postReadProcessing: ".$DB->{postReadProcessing}.": ".$@) if ($@);
+		evalCustomCode($DB->{postReadProcessing},"postReadProcessing");
 	}
 	EAI::File::writeText($File,\@{$process->{data}}) or do {
 		$logger->error("error creating/writing file");
@@ -957,7 +980,7 @@ list of hashes defining specific load processes within the task script. Each has
 
 In the above mentioned hashes can be five categories (sub-hashes): L<DB|/DB>, L<File|/File>, L<FTP|/FTP>, L<process|/process> and L<task|/task>. These allow further parameters to be set for the respective parts of EAI::Wrap (L<EAI::DB>, L<EAI::File> and L<EAI::FTP>), process parameters and task parameters. The parameters are described in detail in section L<CONFIGURATION REFERENCE|/CONFIGURATION REFERENCE>.
 
-The L<process|/process> category is on the one hand used to pass information within each process (data, additionalLookupData, filenames, hadErrors or commandline parameters starting with interactive), on the other hand for additional configurations not suitable for L<DB|/DB>, L<File|/File> or L<FTP|/FTP> (e.g. L<uploadCMD|/uploadCMD>). The L<task|/task> category contains parameters used on the task script level and is therefore only allowed in C<%config> and C<%common>. It contains parameters for skipping, retrying and redoing the whole task script.
+The L<process|/process> category is on the one hand used to pass information within each process (data, additionalLookupData, filenames, hadErrors or custom commandline parameters starting with interactive), on the other hand for additional configurations not suitable for L<DB|/DB>, L<File|/File> or L<FTP|/FTP> (e.g. L<uploadCMD|/uploadCMD>). The L<task|/task> category contains parameters used on the task script level and is therefore only allowed in C<%config> and C<%common>. It contains parameters for skipping, retrying and redoing the whole task script.
 
 The settings in DB, File, FTP and task are "merge" inherited in a cascading manner (i.e. missing parameters are merged, parameters already set below are not overwritten):
 
@@ -996,83 +1019,121 @@ L<getLocalFiles|/getLocalFiles>, L<getFilesFromFTP|/getFilesFromFTP>, L<getFiles
 
 After the first successful run of the task, C<$execute{firstRunSuccess}> is set to prevent any error messages resulting of files having been moved/removed while rerunning the task until the defined planned time (C<task =E<gt> {plannedUntil =E<gt> "HHMM"}>) has been reached.
 
-=item INIT ()
+=item INIT
 
 The INIT procedure is executed at the EAI::Wrap module initialization (when EAI::Wrap is used in the task script) and loads the site configuration, starts logging and reads commandline options. This means that everything passed to the script via command line may be used in the definitions, especially the C<task{interactive.*}> parameters, here the name and the type of the parameter are not checked by the consistency checks (all other parameters not allowed or having the wrong type would throw an error).
 
 =item removeFilesinFolderOlderX
 
-remove files on FTP server being older than a time back (given in day/mon/year in C<remove =E<gt> {removeFolders =E<gt> ["",""], day=E<gt>, mon=E<gt>, year=E<gt>1}>), see L<EAI::FTP::removeFilesOlderX>
+remove files on FTP server being older than a time back (given in day/mon/year in C<remove =E<gt> {removeFolders =E<gt> ["",""], day=E<gt>, mon=E<gt>, year=E<gt>1}>), see L<EAI::FTP::removeFilesOlderX|EAI::FTP/removeFilesOlderX>
 
-=item openDBConn
+=item openDBConn ($)
 
-open a DB connection with the information provided in C<$DB-E<gt>{user}>, C<$DB-E<gt>{pwd}> (these can be provided by the sensitive information looked up using C<$DB-E<gt>{prefix}>) and C<$DB-E<gt>{DSN}> which can be dynamically configured using information from C<$DB> itself, using C<$execute{env}> inside C<$DB-E<gt>{server}{*}>: C<'driver={SQL Server};Server=$DB-E<gt>{server}{$execute{env}};database=$DB-E<gt>{database};TrustedConnection=Yes;'>, also see L<EAI::DB::newDBH|/EAI::DB::newDBH>
+argument $arg (ref to current load or common)
 
-=item openFTPConn
+open a DB connection with the information provided in C<$DB-E<gt>{user}>, C<$DB-E<gt>{pwd}> (these can be provided by the sensitive information looked up using C<$DB-E<gt>{prefix}>) and C<$DB-E<gt>{DSN}> which can be dynamically configured using information from C<$DB> itself, using C<$execute{env}> inside C<$DB-E<gt>{server}{*}>: C<'driver={SQL Server};Server=$DB-E<gt>{server}{$execute{env}};database=$DB-E<gt>{database};TrustedConnection=Yes;'>, also see L<EAI::DB::newDBH|EAI::DB/newDBH>
 
-open a FTP connection with the information provided in C<$FTP-E<gt>{remoteHost}>, C<$FTP-E<gt>{user}>, C<$FTP-E<gt>{pwd}>, C<$FTP-E<gt>{hostkey}>, C<$FTP-E<gt>{privKey}> (these four can be provided by the sensitive information looked up using C<$FTP-E<gt>{prefix}>) and C<$execute{env}>, also see L<EAI::FTP::login|/EAI::FTP::login>
+=item openFTPConn ($)
 
-=item redoFiles
+argument $arg (ref to current load or common)
+
+open a FTP connection with the information provided in C<$FTP-E<gt>{remoteHost}>, C<$FTP-E<gt>{user}>, C<$FTP-E<gt>{pwd}>, C<$FTP-E<gt>{hostkey}>, C<$FTP-E<gt>{privKey}> (these four can be provided by the sensitive information looked up using C<$FTP-E<gt>{prefix}>) and C<$execute{env}>, also see L<EAI::FTP::login|EAI::FTP/login>
+
+=item redoFiles ($)
+
+argument $arg (ref to current load or common)
 
 redo file from redo directory if specified (C<$common{task}{redoFile}> is being set), this is also being called by getLocalFiles and getFilesFromFTP. Arguments are fetched from common or loads[i], using File parameter.
 
-=item getLocalFiles
+=item getLocalFiles ($)
+
+argument $arg (ref to current load or common)
 
 get local file(s) from source into homedir, uses C<$File-E<gt>{filename}>, C<$File-E<gt>{extension}> and C<$File-E<gt>{avoidRenameForRedo}>. Arguments are fetched from common or loads[i], using File parameter.
 
-=item getFilesFromFTP
+=item getFilesFromFTP ($)
+
+argument $arg (ref to current load or common)
 
 get file/s (can also be a glob for multiple files) from FTP into homedir and extract archives if needed. Arguments are fetched from common or loads[i], using File and FTP parameters.
 
-=item getFiles
+=item getFiles ($)
 
-combines above two procedures in a general procedure to get files from FTP or locally. Arguments are fetched from common or loads[i], using File and FTP parameters.
+argument $arg (ref to current load or common)
 
-=item checkFiles
+combines above two procedures in a general procedure to get files from FTP or locally. Arguments are fetched from common or loads[i], using File and FTP parameters. 
+
+All get<*Files*> functions also parse the file into the datastructure process{data}. Custom "hooks" can be defined with L<fieldCode|/fieldCode> and L<lineCode|/lineCode> to modify and enhance the standard mapping defined in format_header. To access the final line data the hash %EAI::File::line can be used (specific fields with $EAI::File::line{<target header column>}). if a field is being replaced using a different name from targetheader, the data with the original header name is placed in %EAI::File::templine. You can also access data from the previous line with %EAI::File::previousline and the previous temp line with %EAI::File::previoustempline.
+
+=item checkFiles ($)
+
+argument $arg (ref to current load or common)
 
 check files for continuation of processing and extract archives if needed. Arguments are fetched from common or loads[i], using File parameter. The processed files are put into process->{filenames}
 
-=item extractArchives
+=item extractArchives ($)
+
+argument $arg (ref to current load or common)
 
 extract files from archive. Arguments are fetched from common or loads[i], using only the process->{filenames} parameter that was filled by checkFiles. 
 
-=item getAdditionalDBData
+=item getAdditionalDBData ($;$)
+
+arguments $arg (ref to current load or common) and optional $refToDataHash
 
 get additional data from DB. Arguments are fetched from common or loads[i], using DB and process parameters. You can also pass an optional ref to a data hash parameter to store the retrieved data there instead of C<$process->{additionalLookupData}>
 
-=item readFileData
+=item readFileData ($)
+
+argument $arg (ref to current load or common)
 
 read data from a file. Arguments are fetched from common or loads[i], using File parameter.
 
-=item dumpDataIntoDB
+=item dumpDataIntoDB ($)
+
+argument $arg (ref to current load or common)
 
 store data into Database. Arguments are fetched from common or loads[i], using DB and File (for emptyOK) parameters.
 
-=item markProcessed
+=item markProcessed ($)
+
+argument $arg (ref to current load or common)
 
 mark files as being processed depending on whether there were errors, also decide on removal/archiving of downloaded files. Arguments are fetched from common or loads[i], using File parameter.
 
-=item writeFileFromDB
+=item writeFileFromDB ($)
+
+argument $arg (ref to current load or common)
 
 create Data-files from Database. Arguments are fetched from common or loads[i], using DB and File parameters.
 
-=item putFileInLocalDir
+=item putFileInLocalDir ($)
+
+argument $arg (ref to current load or common)
 
 put files into local folder if required. Arguments are fetched from common or loads[i], using File parameter.
 
-=item markForHistoryDelete
+=item markForHistoryDelete ($)
+
+argument $arg (ref to current load or common)
 
 mark to be removed or be moved to history after upload. Arguments are fetched from common or loads[i], using File parameter.
 
-=item uploadFileToFTP
+=item uploadFileToFTP ($)
+
+argument $arg (ref to current load or common)
 
 upload files to FTP. Arguments are fetched from common or loads[i], using FTP and File parameters.
 
-=item uploadFileCMD
+=item uploadFileCMD ($)
+
+argument $arg (ref to current load or common)
 
 upload files using an upload command program. Arguments are fetched from common or loads[i], using File and process parameters.
 
-=item uploadFile
+=item uploadFile ($)
+
+argument $arg (ref to current load or common)
 
 combines above two procedures in a general procedure to upload files via FTP or CMD or to put into local dir. Arguments are fetched from common or loads[i], using File and process parameters
 
@@ -1080,17 +1141,21 @@ combines above two procedures in a general procedure to upload files via FTP or
 
 final processing steps for processEnd (cleanup, FTP removal/archiving) or retry after pausing. No context argument as this always depends on all loads and/or the common definition
 
-=item processingPause
+=item processingPause ($)
 
 generally available procedure for pausing processing, argument $pauseSeconds gives the delay
 
 =item moveFilesToHistory (;$)
 
+optional argument $archiveTimestamp
+
 move transferred files marked for moving (filesToMoveinHistory/filesToMoveinHistoryUpload) into history and/or historyUpload folder. Optionally a custom timestamp can be passed.
  
 =item deleteFiles ($)
 
-delete transferred files. The filenames are passed in a ref to array
+argument $filenames, ref to array
+
+delete transferred files given in $filenames
 
 =back
 
@@ -1108,7 +1173,7 @@ parameter category for site global settings, defined in site.config and other as
 
 =item checkLookup
 
-used for logchecker, each entry of the hash defines a log to be checked, defining errmailaddress to receive error mails, errmailsubject, timeToCheck as earliest time to check for existence in log, freqToCheck as frequency of checks (daily/monthly/etc), logFileToCheck as the name of the logfile to check, logcheck as the regex to check in the logfile and logRootPath as the folder where the logfile is found. lookup key: $execute{scriptname} + $execute{addToScriptName}
+ref to datastructure {"scriptname.pl" => {errmailaddress => "",errmailsubject => "",timeToCheck =>"", freqToCheck => "", logFileToCheck => "", logcheck => "",logRootPath =>""},...} used for logchecker, each entry of the hash lookup table defines a log to be checked, defining errmailaddress to receive error mails, errmailsubject, timeToCheck as earliest time to check for existence in log, freqToCheck as frequency of checks (daily/monthly/etc), logFileToCheck as the name of the logfile to check, logcheck as the regex to check in the logfile and logRootPath as the folder where the logfile is found. lookup key: $execute{scriptname} + $execute{addToScriptName}
 
 =item errmailaddress
 
@@ -1124,7 +1189,7 @@ code to be executed during INIT of EAI::Wrap to allow for assignment of config/e
 
 =item folderEnvironmentMapping
 
-Mapping for $execute{envraw} to $execute{env}:
+ref to hash {Test => "Test", Dev => "Dev", "" => "Prod"}, mapping for $execute{envraw} to $execute{env}
 
 =item fromaddress
 
@@ -1132,11 +1197,11 @@ from address for central logcheck/errmail sending, also used as default sender a
 
 =item historyFolder
 
-folders where downloaded files are historized, lookup key as checkLookup, default in "" =>
+ref to hash {"scriptname.pl" => "folder"}, folders where downloaded files are historized, lookup key as in checkLookup, default in "" => "defaultfolder"
 
 =item historyFolderUpload
 
-folders where uploaded files are historized, lookup key as checkLookup, default in "" =>
+ref to hash {"scriptname.pl" => "folder"}, folders where uploaded files are historized, lookup key as in checkLookup, default in "" => "defaultfolder"
 
 =item logCheckHoliday
 
@@ -1148,15 +1213,15 @@ logs to be ignored in central logcheck/errmail sending
 
 =item logRootPath
 
-paths to log file root folders (environment is added to that if non production), lookup key as checkLookup, default in "" =>
+ref to hash {"scriptname.pl" => "folder"}, paths to log file root folders (environment is added to that if non production), lookup key as checkLookup, default in "" => "defaultfolder"
 
 =item redoDir
 
-folders where files for redo are contained, lookup key as checkLookup, default in "" =>
+ref to hash {"scriptname.pl" => "folder"}, folders where files for redo are contained, lookup key as checkLookup, default in "" => "defaultfolder"
 
 =item sensitive
 
-hash lookup for sensitive access information in DB and FTP (lookup keys are set with DB{prefix} or FTP{prefix}), may also be placed outside of site.config; all sensitive keys can also be environment lookups, e.g. hostkey=>{Test => "", Prod => ""} to allow for environment specific setting
+hash lookup table ({"prefix" => {user=>"",pwd =>"",hostkey=>"",privkey =>""},...}) for sensitive access information in DB and FTP (lookup keys are set with DB{prefix} or FTP{prefix}), may also be placed outside of site.config; all sensitive keys can also be environment lookups, e.g. hostkey=>{Test => "", Prod => ""} to allow for environment specific setting
 
 =item smtpServer
 
@@ -1368,15 +1433,15 @@ if files from this load should not be dumped to the database
 
 =item postDumpExecs
 
-done in dumpDataIntoDB after postDumpProcessing and before commit/rollback. doInDB everything in execs if condition is fulfilled 
+array for execs done in dumpDataIntoDB after postDumpProcessing and before commit/rollback: [{execs => ['',''], condition => ''}]. doInDB all execs if condition (evaluated string or anonymous sub: condition => sub {...}) is fulfilled
 
 =item postDumpProcessing
 
-done in dumpDataIntoDB after storeInDB, execute perl code in postDumpProcessing
+done in dumpDataIntoDB after storeInDB, execute perl code in postDumpProcessing (evaluated string or anonymous sub: postDumpProcessing => sub {...})
 
 =item postReadProcessing
 
-done in writeFileFromDB after readFromDB, execute perl code in postReadProcessing
+done in writeFileFromDB after readFromDB, execute perl code in postReadProcessing (evaluated string or anonymous sub: postReadProcessing => sub {...})
 
 =item prefix
 
@@ -1400,7 +1465,7 @@ schemaName used in dumpDataIntoDB/storeInDB, if tableName contains dot the extra
 
 =item server
 
-DB Server in environment hash
+DB Server in environment hash lookup: {Prod => "", Test => ""}
 
 =item tablename
 
@@ -1456,7 +1521,7 @@ the extension of the file to be read (optional, used for redoFile)
 
 =item fieldCode
 
-additional field based processing code: fieldCode => {field1 => 'perl code', ..}, invoked if key equals either header (as in format_header) or targetheader (as in format_targetheader) or invoked for all fields if key is empty {"" => 'perl code'}. set $skipLineAssignment to true (1) if current line should be skipped from data.
+additional field based processing code: fieldCode => {field1 => 'perl code', ..}, invoked if key equals either header (as in format_header) or targetheader (as in format_targetheader) or invoked for all fields if key is empty {"" => 'perl code'}. set $EAI::File::skipLineAssignment to true (1) if current line should be skipped from data. perl code can be an evaluated string or an anonymous sub: field1 => sub {...}
 
 =item filename
 
@@ -1580,7 +1645,7 @@ specify xml parsing
 
 =item lineCode
 
-additional line based processing code, invoked after whole line has been read
+additional line based processing code, invoked after whole line has been read (evaluated string or anonymous sub: lineCode => sub {...})
 
 =item localFilesystemPath
 
@@ -1676,7 +1741,7 @@ queue_size for Net::SFTP::Foreign, if > 1 this causes often connection issues
 
 =item remove
 
-for for removing (archived) files with removeFilesOlderX, all files in removeFolders are deleted being older than day=> days, mon=> months and year=> years
+ref to hash {removeFolders=>[], day=>, mon=>, year=>1} for for removing (archived) files with removeFilesOlderX, all files in removeFolders are deleted being older than day days, mon months and year years
 
 =item remoteDir
 
